-- GLOBAL --
DA = DA or { };
-- MODULE --
local _G = getfenv(0);
setmetatable(DA, {__index = _G});
setfenv(1, DA);
-- LOCAL --
title		= "Dimmed Actions"
version		= {major=1, minor=0, rev=7};--beta
versionFull	= version.major.."."..version.minor.."."..version.rev;
titleFull	= title.." "..versionFull

coreFrame = CreateFrame("Frame");
core = LibStub("AceAddon-3.0"):NewAddon(coreFrame, title, "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0") -- 

-- API Lookups
local ActionHasRange		= ActionHasRange;
local BOOKTYPE_SPELL		= BOOKTYPE_SPELL;
local GetActionInfo			= GetActionInfo;
local GetSpellName			= GetSpellName;
local HasAction				= HasAction;
local IsActionInRange		= IsActionInRange;
local IsAltKeyDown			= IsAltKeyDown;
local IsUsableAction		= IsUsableAction;
local UnitAffectingCombat	= UnitAffectingCombat;
local UnitBuff				= UnitBuff;
local UnitDebuff			= UnitDebuff;
local UnitExists			= UnitExists;
local UnitIsEnemy			= UnitIsEnemy;
local UnitIsEnemy			= UnitIsEnemy;
local UnitIsFriend			= UnitIsFriend;
local pairs					= pairs;
local print					= print;
local table_getn			= table.getn;
local table_insert			= table.insert;
local tostring				= tostring;
--~ local UnitIsEnemy_			= UnitIsEnemy;
local SecureCmdOptionParse_	= SecureCmdOptionParse;
local InCombatLockdown_		= InCombatLockdown;
local GetSpellLink			= GetSpellLink;
local GetTime_				= GetTime;
local GetSpellCooldown_		= GetSpellCooldown;

--Internal settings, don't touch.
BT4	= false; --Is bartender running? this gets set in OnEnable()

myTooltip = CreateFrame("GameTooltip", "DA_ToolTip", UIParent, "GameTooltipTemplate");

--Default Settings
db				= {};
defaultSettings	= {profile = {}}
defaultSettings.profile.updateFrequency	= 0.25;
defaultSettings.profile.dimAlpha		= 0.3;
defaultSettings.profile.showAlt			= true;
defaultSettings.profile.showNoTarget	= true;
defaultSettings.profile.dimCombatOnly	= true;
defaultSettings.profile.dimNoMana	= true;
defaultSettings.profile.dimOutOfRange	= true;
defaultSettings.profile.dimSpellExists	= true;
defaultSettings.profile.readMacros	= false;
defaultSettings.profile.macroFindSpellSpam	= true;
defaultSettings.profile.dimCooldown	= true;


defaultSettings.profile.healingSpellsOnTargetsTarget	= true;


--bars
defaultSettings.profile.barsBar1	= true;
defaultSettings.profile.barsBonusBar1	= true;
defaultSettings.profile.barsMultiBarBottomLeft	= false;
defaultSettings.profile.barsMultiBarBottomRight	= false;
defaultSettings.profile.barsMultiBarRight	= false;
defaultSettings.profile.barsMultiBarLeft	= false;


--BT4 Optinos
defaultSettings.profile.barsBT4Bar1	= true;
defaultSettings.profile.barsBT4Bar2	= false;
defaultSettings.profile.barsBT4Bar3	= false;
defaultSettings.profile.barsBT4Bar4	= false;
defaultSettings.profile.barsBT4Bar5	= false;
defaultSettings.profile.barsBT4Bar6	= false;
defaultSettings.profile.barsBT4Bar7	= false;
defaultSettings.profile.barsBT4Bar8	= false;
defaultSettings.profile.barsBT4Bar9	= false;
defaultSettings.profile.barsBT4Bar10	= false;

local GCDspells = {"Shoot", "Cure Poison", "Purify", "Purge"}--spells I can check to see if we're on global cooldown. Shoot for most classes, Cure Posion for druid, Purify for Pally, Purge for shammy.

healSpells = {--some heal events aren't heal spells, like hunter/warlock heal pet spells.
	["Abolish Disease"] = true,
	["Abolish Poison"] = true,
	["Blessing of Protection"] = true,
	["Chain Heal"] = true,
	["Cleanse"] = true,
	["Cure Disease"] = true,
	["Cure Poison"] = true,
	["Dispel Magic"] = true,
	["First Aid"] = true,
	["Flash Heal"] = true,
	["Flash of Light"] = true,
	["Greater Heal"] = true,
	["Heal"] = true,
	["Healing Touch"] = true,
	["Healing Wave"] = true,
	["Holy Light"] = true,
	["Lesser Heal"] = true,
	["Lesser Healing Wave"] = true,
	["Power Word: Shield"] = true,
	["Purify"] = true,
	["Regrowth"] = true,
	["Rejuvenation"] = true,
	["Remove Curse"] = true,
	["Remove Lesser Curse"] = true,
	["Renew"] = true,
}

barOpts = false; --needed for bar display options.
coreOpts = false;

-- Ace3 Functions
function core:OnInitialize()
	db = LibStub("AceDB-3.0"):New("DA_DB", defaultSettings)
	CoreOptionsTable.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(db)--save option profile or load another chars opts.

	db.RegisterCallback(self, "OnProfileReset", "OnProfileReset")
	
	core:RegisterChatCommand("da", "MySlashProcessorFunc")
	
	local config = LibStub("AceConfig-3.0");
	local dialog = LibStub("AceConfigDialog-3.0");
	
	config:RegisterOptionsTable(title, CoreOptionsTable)
	coreOpts = dialog:AddToBlizOptions(title, "DA "..versionFull)
	
	config:RegisterOptionsTable(title.."Bars", BarOptionsTable)
	barOpts = dialog:AddToBlizOptions(title.."Bars", "Bars", "DA "..versionFull)
	
	config:RegisterOptionsTable(title.."SE", SpellExistOptionsTable)
	barOpts = dialog:AddToBlizOptions(title.."SE", "Spell Exists", "DA "..versionFull)
	
end
function core:MySlashProcessorFunc(input)
	InterfaceOptionsFrame_OpenToCategory(barOpts)
	InterfaceOptionsFrame_OpenToCategory(coreOpts)
end

local regEvents = {
	"ACTIONBAR_SLOT_CHANGED",
--~ 	"SPELL_UPDATE_COOLDOWN",
}
function core:OnEnable()
	for i, event in pairs(regEvents) do 
		core:RegisterEvent(event);
	end
	
	if _G.Bartender4 then
		BT4 = true;
		AddBT4Options()
	end
	
	CollectSpellNameInfo()
	HookActionButtons()
	
	_G.BonusActionBarTexture0:SetAlpha(0)--ugly background frames.
	_G.BonusActionBarTexture1:SetAlpha(0)
	
	if db.profile.dimSpellExists == true then
		AddSpellExistOptions()
	end
end


function core:OnProfileReset(...)
	local whatHappen, info = select(1, ...);

	db.profile.se = nil;
	AddSpellExistOptions()
end

  


function core:OnDisable()
	-- I know when ace addons are disabled their hooks are released, these functions return the actions to 100% alpha.
	if BT4 == true then
		UnhookBar("BT4Bar1")
		UnhookBar("BT4Bar2")
		UnhookBar("BT4Bar3")
		UnhookBar("BT4Bar4")
		UnhookBar("BT4Bar5")
		UnhookBar("BT4Bar6")
		UnhookBar("BT4Bar7")
		UnhookBar("BT4Bar8")
		UnhookBar("BT4Bar9")
		UnhookBar("BT4Bar10")
	else
		UnhookBar("Bar1")
		UnhookBar("BonusBar1")
		UnhookBar("MultiBarBottomLeft")
		UnhookBar("MultiBarBottomRight")
		UnhookBar("MultiBarRight")
		UnhookBar("MultiBarLeft")
	end
end


function GetMouseoverInfo()--	/script DA.GetMouseoverInfo()
	local self = GetMouseFocus();
	print(self:GetName().." - "..self.action)
	if self.action then
		local actionID = self.action
		
--~ 	name, iconTexture, body, local = GetMacroInfo(MacroID)
		local actionType, spellID = GetActionInfo(actionID)
		if actionType == "macro" then
			local body = GetMacroBody(spellID)
			local result, target = SecureCmdOptionParse_(body)
			Debug("GetMouseoverInfo 1", result, target)
		
		
--~ 			local body = GetMacroBody(spellID)
--~ 			local result, target = SecureCmdOptionParse_(body)
--~ 			Debug("GetMouseoverInfo1", result, target)
--~ 			
			
		end
--~ 		print(self:GetName().." - "..self.action, GetActionInfo(self.action))
		
	end
end



--.. " - "..GetMouseFocus().action
--~ /script ChatFrame1:AddMessage(GetMouseFocus():GetParent():GetName().." - "..GetMouseFocus():GetName())
function HookActionButtons()
	if BT4 == true then
		if db.profile.barsBT4Bar1 == true then
			CheckBar("BT4Bar1")
		end
		
		if db.profile.barsBT4Bar2 == true then
			CheckBar("BT4Bar2")
		end
		if db.profile.barsBT4Bar3 == true then
			CheckBar("BT4Bar3")
		end
		if db.profile.barsBT4Bar4 == true then
			CheckBar("BT4Bar4")
		end
		if db.profile.barsBT4Bar5 == true then
			CheckBar("BT4Bar5")
		end
		if db.profile.barsBT4Bar6 == true then
			CheckBar("BT4Bar6")
		end
		if db.profile.barsBT4Bar7 == true then
			CheckBar("BT4Bar7")
		end
		if db.profile.barsBT4Bar8 == true then
			CheckBar("BT4Bar8")
		end
		if db.profile.barsBT4Bar9 == true then
			CheckBar("BT4Bar9")
		end
		if db.profile.barsBT4Bar10 == true then
			CheckBar("BT4Bar10")
		end
	else

		if db.profile.barsBar1 == true then
			for i=1, NUM_ACTIONBAR_BUTTONS, 1 do
				HookActionButton(_G["ActionButton"..i], "Bar1", i)
			end
		end
		
		if db.profile.barsBonusBar1 == true then
			for i=1, NUM_ACTIONBAR_BUTTONS, 1 do
				HookActionButton(_G["BonusActionButton"..i], "BonusBar1", i)
			end
		end
		
		
		if db.profile.barsMultiBarBottomRight == true then
			for i=1, NUM_ACTIONBAR_BUTTONS, 1 do
				HookActionButton(_G["MultiBarBottomRightButton"..i], "MultiBarBottomRight", i)
			end
		end
		
		if db.profile.barsMultiBarBottomLeft == true then
			for i=1, NUM_ACTIONBAR_BUTTONS, 1 do
				HookActionButton(_G["MultiBarBottomLeftButton"..i], "MultiBarBottomLeft", i)
			end
		end
		
		if db.profile.barsMultiBarRight == true then
			for i=1, NUM_ACTIONBAR_BUTTONS, 1 do
				HookActionButton(_G["MultiBarRightButton"..i], "MultiBarRight", i)
			end
		end
		if db.profile.barsMultiBarLeft == true then
			for i=1, NUM_ACTIONBAR_BUTTONS, 1 do
				HookActionButton(_G["MultiBarLeftButton"..i], "MultiBarLeft", i)
			end
		end
		
		
	end
end

function CheckBar(barName)--	/script
	local bar = _G[barName];--"BT4Bar1"
	
	
	local kids = { bar:GetChildren() };
	
	for _, child in ipairs(kids) do
	
		if child:GetName():find("Button") then
--~ 			print(child:GetName())
			HookActionButton(child, "bars"..barName, 0)
		end
		
		--
	end
end

spellNames = {};--spellname to IDs.

local GetMacroInfo_ = GetMacroInfo;
hookedButtons = {};
function HookActionButton(buttonFrame, bar, id)
--~ 	local buttonFrame = _G[buttonName];
	if buttonFrame then
		hookedButtons[bar] = hookedButtons[bar] or {};
		if id == 0 then
			id = table.getn(hookedButtons[bar]) + 1;
		end
		hookedButtons[bar][id] = buttonFrame
		
--~ 		Debug("HookActionButton",bar, id)

		if buttonFrame:HasScript("OnUpdate") and not core:IsHooked(buttonFrame, "OnUpdate") then
			buttonFrame.daLastUpdate = 0;
			core:HookScript(buttonFrame, "OnUpdate", ActionButton_OnUpdate);
			
--~ 			Debug("HookActionButton", buttonFrame:GetName())
		end
		
		if db.profile.macroFindSpellSpam == true then
			if buttonFrame.action then
				local actionID = buttonFrame.action;
				local actionType, spellID = GetActionInfo(actionID)
				if actionType == "macro" then
					
					local name, _, body = GetMacroInfo_(spellID)

					local result, target = SecureCmdOptionParse_(body)
					
					if result and result ~= "" and not result:find("\n") then
						if result:find("!") then
							result = result:gsub("!","")
						end
						if not spellNames[result] then
							print("[Dimmed Actions] Cannot find "..result.." on any bars, macro '"..name.."' won't be dimmed.")
						end
					end
				end
			end
		end
	end
end

function UnhookBar(bar)
	if hookedButtons[bar] then
		for _, buttonFrame in pairs(hookedButtons[bar]) do 
			buttonFrame:SetAlpha(1);
			if core:IsHooked(buttonFrame, "OnUpdate") then
				core:Unhook(buttonFrame, "OnUpdate")
			end
		end
	end
end

function ActionButton_OnUpdate(self, elapsed)
	self.daLastUpdate = self.daLastUpdate + elapsed;
	if self.daLastUpdate >= db.profile.updateFrequency then 
		self.daLastUpdate = 0;
		
		UpdateActionButton(self)
--~ 		print("ActionButton_OnUpdate "..self:GetName())
	end
end

--~ actionIDs = {}


------------------------------------------------------------------------------
function CollectSpellNameInfo()												--	/script DA.CollectSpellNameInfo()
-- I save actionID and spellID indexed by spellname to help with macros.	--
-- Since macros don't tell us their spellID or a actionID, I check the 		--
-- spellname to our list and use the known spellID and actionID.			--
------------------------------------------------------------------------------
	local actionID, actionType, spellID, subType, spellRank, spellName;
	for b=0, 9 do
		for s = 1, 12 do
			actionID = s+(b*12);
			if HasAction(actionID) then -- and ActionHasRange_(actionID)
				actionType, spellID, subType  = GetActionInfo(actionID)
				if actionType == "spell" and subType  == "spell" and spellID and spellID > 0 then
					spellName, spellRank = GetSpellName( spellID, BOOKTYPE_SPELL );
					if spellName and not spellNames[spellName] then
						local targetSpell, hasCooldown = GetSpellDetails(spellName, spellRank)
						spellNames[spellName] = {aID=actionID, sID=spellID, targetSpell=targetSpell, hasCooldown = false}
						
						if spellName ~= "Shoot" then
							spellNames[spellName].hasCooldown = hasCooldown;
						end
					end
				end
			end
		end
	end
end


--~ function core:SPELL_UPDATE_COOLDOWN(_, ...)
--~ 	Debug("SPELL_UPDATE_COOLDOWN", ...)
--~ end

function core:ACTIONBAR_SLOT_CHANGED(_, ...)
	local actionID  = select(1, ...)-- ***
	for spellName, data in pairs(spellNames) do 
		if data.aID == actionID then
			spellNames[spellName].aID = nil;
			break;
		end
	end
	if HasAction(actionID) then -- and ActionHasRange_(actionID)
		actionType, spellID, subType  = GetActionInfo(actionID)
		if actionType == "spell" and subType  == "spell" and spellID and spellID > 0 then
			spellName = GetSpellName( spellID, BOOKTYPE_SPELL );
			if spellName and spellNames[spellName] then
				local targetSpell, hasCooldown = GetSpellDetails(spellName, spellRank)
				spellNames[spellName].aID = actionID;
			end
		end
	end
end


--
function UpdateActionButton(self)
	local actionID = self.action
--~ 	Debug("UpdateActionButton", self:GetName(), actionID)
	
--~ 	if true then
--~ 		self:SetAlpha(math.random(10)/10);
--~ 		return;
--~ 	end
	
	if BT4 == false and self:GetName():find("BonusActionButton") then
		_G["ActionButton"..self:GetID()]:SetAlpha(0)--hide the button behind it.
	end
	
	if db.profile.showAlt == true then
		if IsAltKeyDown() then
			self:SetAlpha(1);
--~ 			Debug("UpdateActionButton 1", self:GetName(), actionID)
			return;
		end
	end
	
--~ 	
	
	
	if db.profile.showNoTarget == true then
		if not UnitExists("target") then
			self:SetAlpha(1);
--~ 			Debug("UpdateActionButton 2", self:GetName(), actionID)
			return;
		end
	end
	
	if db.profile.dimCombatOnly == true then
		if not InCombatLockdown_() then
			self:SetAlpha(1);
--~ 			Debug("UpdateActionButton 3", self:GetName(), actionID)
			return;
		end
	end
	
	local targetID = "target";
	
	local actionType, spellID = GetActionInfo(actionID)
	if actionType == "macro" then
		if db.profile.readMacros == true then
			local body = GetMacroBody(spellID)
			local result, target = SecureCmdOptionParse_(body)
			
			if target and target ~= targetID then --and target ~= "" and 
				targetID = target;
			end
			if result and result ~= "" then
				if result:find("!") then
					result = result:gsub("!","")
				end
				
	--~ 			Debug("UpdateActionButton 2",self:GetName(), mType, result)
				if spellNames[result] then
	--~ 					Debug("UpdateActionButton 3", "Found action ID", result, actionID)
					actionID = spellNames[result].aID or actionID;
					actionType = "spell"
					spellID = spellNames[result].sID;
				else
					self:SetAlpha(1);
					return;
				end
			else
				self:SetAlpha(1);
				return;
			end
		else
			self:SetAlpha(1);
			return;
		end
	end
	
	
	local isUsable, notEnoughMana = IsUsableAction(actionID)
	if db.profile.dimNoMana == true then
		if notEnoughMana then
			self:SetAlpha(db.profile.dimAlpha);
--~ 			Debug("UpdateActionButton 5", self:GetName(), actionID)
			return;
		end
	end

	
	if db.profile.dimOutOfRange == true then
		if ActionHasRange(actionID) and IsActionInRange(actionID,targetID) ~= 1 then
			self:SetAlpha(db.profile.dimAlpha);
--~ 			Debug("UpdateActionButton 5", self:GetName(), actionID)
			return;
		end
	end
	if actionType == "spell" then
		local spellName = GetSpellName( spellID, BOOKTYPE_SPELL );
		
		if db.profile.dimCooldown == true and spellNames[spellName].hasCooldown == true then
			local start, duration, enabled = GetSpellCooldown_(spellName);
			if start and start > 0 then --and ( start > 0 and duration > 0) then
			
				
--~ 				local wait = start + duration - GetTime_();
--~ 				Debug(spellName, start, duration, wait)
			
			
				if start ~= GetGCD() then
					self:SetAlpha(db.profile.dimAlpha);
					return;
				end
			end
				
		end
		
		if db.profile.dimSpellExists == true and UnitExists(targetID) then

			if db.profile.se[spellName] == true then
				if db.profile.healingSpellsOnTargetsTarget == true and healSpells[spellName] then
					if not UnitIsFriend("player",targetID) then
						if UnitExists("targettarget") then
							targetID = "targettarget";
						else
							targetID = "player";
						end
					end
				end

				if IsBuffOn(spellName, targetID) then
					self:SetAlpha(db.profile.dimAlpha);
					return;
				end
			end

		end
	end

--~ 	Debug("UpdateActionButton 7", self:GetName(), actionID)
	self:SetAlpha(1);
end


function GetGCD()--	/script DA.GetGCD()
	for i=1, table.getn(GCDspells) do 
		if spellNames[GCDspells[i]] then
			local start, duration, enabled = GetSpellCooldown_(GCDspells[i]); --spellNames["Shoot"].aID
			if start and ( start > 0 and duration > 0) then
--~ 				local wait = start + duration - GetTime_();
--~ 				Debug("GetGCD", start, duration, wait)
--~ 				return wait;

				return start;
			end
		end
	end
	return 0;
end

function IsBuffOn(spellName, unitID)
	local name, unitCaster;
	
	if UnitIsFriend("player", unitID) then 
		for i = 1, 40 do
			name, _, _, _, _, _, _, unitCaster = UnitBuff(unitID, i);
			if not name then break; end
			if name == spellName then
				if unitCaster and unitCaster == "player" then
					return true;
				end
			end
		end
	else--if UnitIsEnemy_("player",unitID) then
		for i = 1, 40 do
			name, _, _, _, _, _, _, unitCaster = UnitDebuff(unitID, i)
			if not name then break; end
			if name == spellName then
				if unitCaster and unitCaster == "player" then
					return true;
				end
			end
		end
	end
	return nil;
end



local strDebugFrom		= "|cffffff00[%s]|r" --Yellow function name. help pinpoint where the debug msg is from.
local strWhiteBar		= "|cffffff00 || |r" -- a white bar to seperate the debug info.
function Debug(from, ...)--	/script V.Debug("hey","bye")
	local tbl  = {...}--select(1, ...)
	local msg = tostring(tbl[1])
	
	for i=2,table_getn(tbl) do 
		msg = msg..strWhiteBar..tostring(tbl[i]);
	end
	
	print(strDebugFrom:format(from).." "..msg)
end

-- GUI settings
CoreOptionsTable = {
	name = title,
	type = "group",
	childGroups = "tab",

	args = {
		core={
			name = "Core",
			type = "group",
			order = 1,
			args={}
		},
	},
}

CoreOptionsTable.args.core.args.enable = {
	type = "toggle",	order	= 1,
	name	= "Enable",
	desc	= "Enables / Disables the addon",
	set = function(info,val) 
		if val == true then
			core:Enable();
		else
			core:Disable();
		end
	end,
	get = function(info) return core:IsEnabled() end
}

CoreOptionsTable.args.core.args.updateFrequency = {
	type 		= "range",	 order		= 2,
	name 		= "Update Frequency",
	desc 		= "|cff00ff00Delay between each action update.|r",
	min			= 0.1,
	max			= 1,
	step		= 0.05,
	set = function(info,val) 
		db.profile.updateFrequency = val;
	end,
	get = function(info) return db.profile.updateFrequency end
}

CoreOptionsTable.args.core.args.dimAlpha = {
	type 		= "range",	 order		= 3,
	name 		= "Dim Alpha",
	desc 		= "|cff00ff00How dim you wan't actions to become.|r",
	min			= 0.0,
	max			= 1, -- 10%
	step		= 0.05,
	isPercent	= true,
	set = function(info,val) 
		db.profile.dimAlpha = val;
	end,
	get = function(info) return db.profile.dimAlpha end
}


CoreOptionsTable.args.core.args.showAlt = {
	type = "toggle",	order	= 4,

	name = "Show Alt-Keypress",
	desc = "|cff00ff00Show all actions when pressing Alt.|r",

	set = function(info,val) 
		db.profile.showAlt = val;
	end,
	get = function(info) return db.profile.showAlt end
}

CoreOptionsTable.args.core.args.showNoTarget = {
	type = "toggle",	order	= 4,

	name = "Show No Target",
	desc = "|cff00ff00Show all actions when you have no target.|r",

	set = function(info,val) 
		db.profile.showNoTarget = val;
	end,
	get = function(info) return db.profile.showNoTarget end
}

CoreOptionsTable.args.core.args.dimCombatOnly = {
	type = "toggle",	order	= 5,

	name = "In Combat Only",
	desc = "|cff00ff00Only dim actions when in combat.|r",

	set = function(info,val) 
		db.profile.dimCombatOnly = val;
	end,
	get = function(info) return db.profile.dimCombatOnly end
}

CoreOptionsTable.args.core.args.dimNoMana = {
	type = "toggle",	order	= 6,

	name = "Dim No Mana",
	desc = "|cff00ff00Dim actions you don't have the mana/energy/rage for.|r",

	set = function(info,val) 
		db.profile.dimNoMana = val;
	end,
	get = function(info) return db.profile.dimNoMana end
}

CoreOptionsTable.args.core.args.dimOutOfRange = {
	type = "toggle",	order	= 7,

	name = "Dim out of range",
	desc = "|cff00ff00Dim actions that are out of range of your target.|r",

	set = function(info,val) 
		db.profile.dimOutOfRange = val;
	end,
	get = function(info) return db.profile.dimOutOfRange end
}

CoreOptionsTable.args.core.args.dimSpellExists = {
	type = "toggle",	order	= 8,

	name = "Dim Spell Exists",
	desc = "|cff00ff00Dim actions if spell exists on target.|r",

	set = function(info,val) 
		db.profile.dimSpellExists = val;
		
		if val == true then
			AddSpellExistOptions()
		else
			for opt in pairs(SpellExistOptionsTable.args) do 
				SpellExistOptionsTable.args[opt] = nil;
			end
		end
	end,
	get = function(info) return db.profile.dimSpellExists end
}
CoreOptionsTable.args.core.args.healingSpellsOnTargetsTarget = {
	type = "toggle",	order	= 9,

	name = "Healing Targets Target",
	desc = "|cff00ff00For healign spells, If your target's hostile, uses your target's target for Spell Exists.\n(Or self if targettarget doesn't exist)|r",

	set = function(info,val) 
		db.profile.healingSpellsOnTargetsTarget = val;
	end,
	get = function(info) return db.profile.healingSpellsOnTargetsTarget end
}


CoreOptionsTable.args.core.args.readMacros = {
	type = "toggle",	order	= 10,

	name = "Read Macros",
	desc = "|cff00ff00Attempt to dim macros.\nPlease put any spells the macro calls on your actionbar (doesn't matter which bar)|r",

	set = function(info,val) 
		db.profile.readMacros = val;
	end,
	get = function(info) return db.profile.readMacros end
}

CoreOptionsTable.args.core.args.macroFindSpellSpam = {
	type = "toggle",	order	= 11,

	name = "Spell Missing Spam",
	desc = "|cff00ff00For DA to work on macros, the spells within the macros need to be on a bar somewhere. Enabling this will spam you on login if those spells are missing.|r",

	set = function(info,val) 
		db.profile.macroFindSpellSpam = val;
	end,
	get = function(info) return db.profile.macroFindSpellSpam end
}

CoreOptionsTable.args.core.args.dimCooldown = {
	type = "toggle",	order	= 12,

	name = "Dim Cooldown",
	desc = "|cff00ff00Dim spells on cooldown.|r",

	set = function(info,val) 
		db.profile.dimCooldown = val;
	end,
	get = function(info) return db.profile.dimCooldown end
}


--Bars
-- GUI settings
BarOptionsTable = {
	name = title,
	type = "group",
	childGroups = "tab",

	args = {
		defaultBars={
			name = "Blizzard Bars",
			type = "group",
			order = 1,
			args={}
		},
	},
}


BarOptionsTable.args.defaultBars.args.barsBar1 = {
	type = "toggle",	order	= 1,

	name = "Action Bar 1",
	desc = "|cff00ff00Work on the default action bar.|r",

	set = function(info,val) 
		db.profile.barsBar1 = val;
		if val == true then
			HookActionButtons()
		else
			UnhookBar("Bar1")
		end
	end,
	get = function(info) return db.profile.barsBar1 end
}
BarOptionsTable.args.defaultBars.args.barsBonusBar1 = {
	type = "toggle",	order	= 2,

	name = "Bonus Bar",
	desc = "|cff00ff00Work on the default bonus bar. (used when stealthing)|r",

	set = function(info,val) 
		db.profile.barsBonusBar1 = val;
		if val == true then
			HookActionButtons()
		else
			UnhookBar("BonusBar1")
		end
	end,
	get = function(info) return db.profile.barsBonusBar1 end
}


BarOptionsTable.args.defaultBars.args.barsMultiBarBottomLeft = {
	type = "toggle",	order	= 2,

	name = "Multi Bar Bottom Left",
	desc = "|cff00ff00Bars on your bottom left (above your main action bar)|r",

	set = function(info,val) 
		db.profile.barsMultiBarBottomLeft = val;
		if val == true then
			HookActionButtons()
		else
			UnhookBar("MultiBarBottomLeft")
		end
	end,
	get = function(info) return db.profile.barsMultiBarBottomLeft end
}

BarOptionsTable.args.defaultBars.args.barsMultiBarBottomRight = {
	type = "toggle",	order	= 3,

	name = "Multi Bar Bottom Right",
	desc = "|cff00ff00Bars on your bottom right (above your bags)|r",

	set = function(info,val) 
		db.profile.barsMultiBarBottomRight = val;
		if val == true then
			HookActionButtons()
		else
			UnhookBar("MultiBarBottomRight")
		end
	end,
	get = function(info) return db.profile.barsMultiBarBottomRight end
}



BarOptionsTable.args.defaultBars.args.barsMultiBarRight = {
	type = "toggle",	order	= 4,

	name = "Right Bar",
	desc = "|cff00ff00Bar on the right side of your screen.|r",

	set = function(info,val) 
		db.profile.barsMultiBarRight = val;
		if val == true then
			HookActionButtons()
		else
			UnhookBar("MultiBarRight")
		end
	end,
	get = function(info) return db.profile.barsMultiBarRight end
}

BarOptionsTable.args.defaultBars.args.barsMultiBarLeft = {
	type = "toggle",	order	= 5,

	name = "Right Bar 2",
	desc = "|cff00ff002nd bar on the right side of your screen.|r",

	set = function(info,val) 
		db.profile.barsMultiBarLeft = val;
		if val == true then
			HookActionButtons()
		else
			UnhookBar("MultiBarLeft")
		end
	end,
	get = function(info) return db.profile.barsMultiBarLeft end
}

function AddBT4Options()
--~ 	BarOptionsTable.args.defaultBars = nil;
	
	
	--disabled
	
	for opt in pairs(BarOptionsTable.args.defaultBars.args) do 
		BarOptionsTable.args.defaultBars.args[opt].disabled = true;
	end
	
	BarOptionsTable.args.BT4Bars = {
		name = "Bartender Bars",
		type = "group",
		order = 2,
		args={}
	}
	
	BarOptionsTable.args.BT4Bars.args.barsBT4Bar1 = {
		type = "toggle",	order	= 1,
	
		name = "Bar 1",
		desc = "|cff00ff00Work on Bartender bar 1|r",
	
		set = function(info,val) 
			db.profile.barsBT4Bar1 = val;
			if val == true then
				HookActionButtons()
			else
				UnhookBar("BT4Bar1")
			end

		end,
		get = function(info) return db.profile.barsBT4Bar1 end
	}
	
	
	BarOptionsTable.args.BT4Bars.args.barsBT4Bar2 = {
		type = "toggle",	order	= 2,
	
		name = "Bar 2",
		desc = "|cff00ff00Work on Bartender bar 2|r",
	
		set = function(info,val) 
			db.profile.barsBT4Bar2 = val;
			if val == true then
				HookActionButtons()
			else
				UnhookBar("BT4Bar2")
			end
		end,
		get = function(info) return db.profile.barsBT4Bar2 end
	}
	
	BarOptionsTable.args.BT4Bars.args.barsBT4Bar3 = {
		type = "toggle",	order	= 3,
	
		name = "Bar 3",
		desc = "|cff00ff00Work on Bartender bar 3|r",
	
		set = function(info,val) 
			db.profile.barsBT4Bar3 = val;
			if val == true then
				HookActionButtons()
			else
				UnhookBar("BT4Bar3")
			end
		end,
		get = function(info) return db.profile.barsBT4Bar3 end
	}
	BarOptionsTable.args.BT4Bars.args.barsBT4Bar4 = {
		type = "toggle",	order	= 4,
	
		name = "Bar 4",
		desc = "|cff00ff00Work on Bartender bar 4|r",
	
		set = function(info,val) 
			db.profile.barsBT4Bar4 = val;
			if val == true then
				HookActionButtons()
			else
				UnhookBar("BT4Bar4")
			end
		end,
		get = function(info) return db.profile.barsBT4Bar4 end
	}
	BarOptionsTable.args.BT4Bars.args.barsBT4Bar5 = {
		type = "toggle",	order	= 5,
	
		name = "Bar 5",
		desc = "|cff00ff00Work on Bartender bar 5|r",
	
		set = function(info,val) 
			db.profile.barsBT4Bar5 = val;
			if val == true then
				HookActionButtons()
			else
				UnhookBar("BT4Bar5")
			end
		end,
		get = function(info) return db.profile.barsBT4Bar5 end
	}
	BarOptionsTable.args.BT4Bars.args.barsBT4Bar6 = {
		type = "toggle",	order	= 6,
	
		name = "Bar 6",
		desc = "|cff00ff00Work on Bartender bar 6|r",
	
		set = function(info,val) 
			db.profile.barsBT4Bar6 = val;
			if val == true then
				HookActionButtons()
			else
				UnhookBar("BT4Bar6")
			end
		end,
		get = function(info) return db.profile.barsBT4Bar6 end
	}
	BarOptionsTable.args.BT4Bars.args.barsBT4Bar7 = {
		type = "toggle",	order	= 7,
	
		name = "Bar 7",
		desc = "|cff00ff00Work on Bartender bar 7|r",
	
		set = function(info,val) 
			db.profile.barsBT4Bar7 = val;
			if val == true then
				HookActionButtons()
			else
				UnhookBar("BT4Bar7")
			end
		end,
		get = function(info) return db.profile.barsBT4Bar7 end
	}
	BarOptionsTable.args.BT4Bars.args.barsBT4Bar8 = {
		type = "toggle",	order	= 8,
	
		name = "Bar 8",
		desc = "|cff00ff00Work on Bartender bar 8|r",
	
		set = function(info,val) 
			db.profile.barsBT4Bar8 = val;
			if val == true then
				HookActionButtons()
			else
				UnhookBar("BT4Bar8")
			end		end,
		get = function(info) return db.profile.barsBT4Bar8 end
	}
	BarOptionsTable.args.BT4Bars.args.barsBT4Bar9 = {
		type = "toggle",	order	= 9,
	
		name = "Bar 9",
		desc = "|cff00ff00Work on Bartender bar 9|r",
	
		set = function(info,val) 
			db.profile.barsBT4Bar9 = val;
			if val == true then
				HookActionButtons()
			else
				UnhookBar("BT4Bar9")
			end
		end,
		get = function(info) return db.profile.barsBT4Bar9 end
	}
	BarOptionsTable.args.BT4Bars.args.barsBT4Bar10 = {
		type = "toggle",	order	= 10,
	
		name = "Bar 10",
		desc = "|cff00ff00Work on Bartender bar 10|r",
	
		set = function(info,val) 
			db.profile.barsBT4Bar10 = val;
			if val == true then
				HookActionButtons()
			else
				UnhookBar("BT4Bar10")
			end
		end,
		get = function(info) return db.profile.barsBT4Bar10 end
	}
end

SpellExistOptionsTable = {
	name = "Spell Exists",
	type = "group",
	childGroups = "tab",

	args = {},
}

SpellExistOptionsTable.args.what = {
	type = "description",
	name = "DA guesses which spells you want to check for on your target but it's not 100% perfect, select any extra dots/hots you want to look for on your target.",
	width = "full",
	order = 1,
}


local table_sort = table.sort;
local table_insert = table.insert;
function AddSpellExistOptions()--	/script DA.AddSpellExistOptions()
	local spells = {};
	for sN in pairs(spellNames) do 
		table_insert(spells, sN);
	end
	table_sort(spells, function(a,b) 
		if(a and b) then 
			return a < b;
		end 
	end)

--~ 	defaultSettings.profile = defaultSettings.profile or {};
	defaultSettings.profile.se = defaultSettings.profile.se or {};
	
	db.profile = db.profile or {};
	db.profile.se = db.profile.se or {};
	
--~ 	SpellExistOptionsTable.args = {};--reset incase user's reenabled addon.
	local index = 2;
	local spellName;
	for i=1, table.getn(spells) do 
		spellName = spells[i];
		
--~ 		Debug("AddSpellExistOptions", spellName, spellNames[spellName].targetSpell)
		
		defaultSettings.profile.se[spellName] = spellNames[spellName].targetSpell;
		if db.profile.se[spellName] == nil then
			db.profile.se[spellName] = spellNames[spellName].targetSpell;
		end
		
		SpellExistOptionsTable.args[spellName] = {
			type = "toggle",	order	= index,
		
			name = spellName,
			desc = "|cff00ff00Check if "..spellName.." exists on target.|r",
		
			set = function(info,val) 
				db.profile.se[info[1]] = val;
				
			end,
			get = function(info) return db.profile.se[info[1]] end
		}
		
		index = index + 1;
	end
	
end

local targetSpells = {--spells that can be cast on other people.
	["the target"] = true,
	["the enemy"] = true,
	["enemy target"] = true,
	["friendly target"] = true,
	["friendly party"] = true,
}

function GetSpellDetails(spellName, spellRank)--	/script DA.GetSpellDetails(339, "Wrath")
	if ( not myTooltip:IsShown() ) then
		myTooltip:SetOwner(UIParent, "ANCHOR_NONE");
	end
	local link = GetSpellLink(spellName, spellRank)
	if not link then
		return false;
	end
	
	local _, _, realid = link:find("Hspell:(%d+)")
	
	myTooltip:SetHyperlink("spell:"..realid)
	myTooltip:Show()

	
	local otherSpell = false;
	local hasCooldown = false;
	local lines = myTooltip:NumLines()
	if lines > 0 then
		local qText = (_G["DA_ToolTipTextLeft"..lines]:GetText() or "")
		for tText in pairs(targetSpells) do 
--~ 			if qText:find("sec") and qText:find(tText) then
			if (qText:find("over") or qText:find("every") or qText:find("sec")) and qText:find(tText) then
--~ 				Debug(spellName, "can be cast on others.")
				otherSpell = true;
				break;
			end
		end

		
		if lines >= 3 then
			for i=1, lines do 
				local qText = (_G["DA_ToolTipTextRight"..i]:GetText() or "")
				if qText:find("cooldown") then
					hasCooldown = true;
					break;
				end
			end

		end
	end
	return otherSpell, hasCooldown;
end
